import {
  CacheType,
  ChatInputCommandInteraction,
  Client,
  IntentsBitField,
  Interaction,
  Routes,
  SlashCommandBuilder,
} from "discord.js";
import { REST } from "@discordjs/rest";
import config from "config";
import PrismaService from "Services/prisma.service";
import getCommands from "Components/commands.component";
import getQuoteModal from "Components/quote-modal.component";
import getActionsComponent from "Components/actions.component";

export default class DiscordService {
  rest: REST;
  client: Client<boolean>;
  private prismaService: PrismaService;

  constructor() {
    this.rest = new REST({ version: "10" }).setToken(config.discord.token);
    this.prismaService = new PrismaService();

    this.client = new Client({ intents: [IntentsBitField.Flags.Guilds] });
  }

  async init() {
    try {
      await this.rest.put(
        Routes.applicationCommands(config.discord.applicationId),
        {
          body: getCommands(),
        }
      );
      console.log("Successfully added commands");

      await this.client.login(config.discord.token);
    } catch (e) {
      console.error(e);
    }
  }

  async handleQuote(interaction: Interaction<CacheType>) {
    let content: string | undefined = undefined;

    if (interaction.isModalSubmit()) {
      content = interaction.fields.getTextInputValue("contentInput");
    } else if (interaction.isChatInputCommand()) {
      content = interaction.options.getString("content") || "";
    }

    if (content) {
      await this.createQuote({ content }, interaction);
    }
  }

  async handleQuoteModal(interaction: Interaction<CacheType>) {
    const modal = getQuoteModal();

    if (interaction.isButton() || interaction.isChatInputCommand()) {
      try {
        await interaction.showModal(modal);
      } catch (e) {
        console.error(e);
      }
    }
  }

  async handleActions(interaction: Interaction<CacheType>) {
    if (interaction.isChatInputCommand()) {
      if (interaction.options.getSubcommand() === "send") {
        const channel = interaction.channel;

        if (channel?.isTextBased()) {
          const actions = getActionsComponent();

          await interaction.reply({
            content: "Choose an action!",
            components: [actions as any],
          });
        }
      }
    }
  }

  async createQuote(
    { content }: { content: string },
    interaction: Interaction
  ) {
    // find the channel with configured id
    const channel = this.client.channels.cache.find(
      (ch) => ch.id === config.discord.channelId
    );

    if (channel?.isTextBased()) {
      // send quote content as message
      await this.prismaService.client.message.create({
        data: {
          content,
          userName: interaction.user.username,
        },
      });

      await channel.send(content);

      if (interaction.isChatInputCommand() || interaction.isModalSubmit()) {
        try {
          await interaction.reply({
            content: "Completed!",
            ephemeral: true,
          });
        } catch (e) {
          console.error(e);
        }
      }

      return;
    }
  }
}
